package org.relaxng.xpathlib;

import org.relaxng.datatype.*;
import org.relaxng.datatype.helpers.StreamingValidatorImpl;

abstract class DatatypeImpl implements Datatype {
	
	protected DatatypeImpl() {}
	
	protected XPathParser parser = new XPathParser(new java.io.StringReader(""));
	
	public boolean isValid( String literal, ValidationContext context ) {
		try {
			synchronized(this) {
				parser.context = context;
				parser.ReInit(new java.io.StringReader(literal));
				run(parser);	// test
			}
			return true;
		} catch(Throwable t) {
			return false;
		}
	}
	
	// this method will throw something if it is invalid.
	protected abstract void run( XPathParser parser ) throws Exception;
	
	public void checkValid( String literal, ValidationContext context ) throws DatatypeException {
		try {
			synchronized(this) {
				parser.context = context;
				parser.ReInit(new java.io.StringReader(literal));
				run(parser);	// test
			}
			return;	// ok.
		} catch( ParseException pe ) {
			// if JavaCC gave us the error diagnosis, use it.
			throw new DatatypeException(
				pe.currentToken.endColumn, pe.getMessage() );
		} catch(Throwable t) {
			// otherwise, abandon the diagnosis.
			throw new DatatypeException();
		}
	}
	
	public DatatypeStreamingValidator createStreamingValidator( ValidationContext context ) {
		// just use StreamingValidatorImpl.
		// JavaCC doesn't support "push"-type parser.
		return new StreamingValidatorImpl(this,context);
	}
	
	public Object createValue( String literal, ValidationContext context ) {
		// you cannot return a value if the specified literal is invalid.
		if(isValid(literal,context))	return literal;
		else							return null;
	}
	
	/*
		This datatype uses the equality of String as the equality of two values.
	*/
	
	public boolean sameValue( Object value1, Object value2 ) {
		if( value1==null || value2==null )	return false;
		return value1.equals(value2);
	}
	
	public int valueHashCode( Object o ) {
		return o.hashCode();
	}
}
